/*-
 * Copyright (c) 2004 - 2009 CTPP Team
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 4. Neither the name of the CTPP Team nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *      VMOpcodes.h
 *
 * $CTPP$
 */
#ifndef _VM_OPCODES_H__
#define _VM_OPCODES_H__ 1

#include "CTPP2Types.h"

// Version 1
#define VM_OPCODE_VERSION  0x00000002

/**
  @file VMOpcodes.h
  @brief Virtual machine operation codes
*/

// Stop machine ////////////////////////////////////////////////////////////////////////////////////
#define HLT              0xFFFFFFFF // Halt
#define BRK              0x7FFFFFFF // Breakpoint
#define NOP              0x3FFFFFFF // No operation

// Instructions //////// 0x---X---- ////////////////////////////////////////////////////////////////
#define SYSCALL          0x00010000 // System call
#define CALL             0x00020000 // Procedure call
#define CALLNAME         0x00020100 // Call procedure by name
#define CALLIND          0x00020200 // Call procedure by name given in regster or stack (indirect)
#define RET              0x00030000 // Return from procedure
#define JMP              0x00040000 // Unconditional jump

// Stack operations //// 0x---X---- ////////////////////////////////////////////////////////////////
#define PUSH             0x00050000 // Push value into stack
#define POP              0x00060000 // Pop value from stack

// Clear register ////// 0x---X---- ////////////////////////////////////////////////////////////////
#define CLEAR            0x00070000 // clear register

// Output data ///////// 0x---X---- ////////////////////////////////////////////////////////////////
#define OUTPUT           0x00080000 // Direct output a value to output collector

// Output data ///////// 0x---X---- ////////////////////////////////////////////////////////////////
#define REPLACE          0x00090000 // Replace variable in stack or register

// Arithmetic ops. ///// 0x--X----- ////////////////////////////////////////////////////////////////
#define ADD              0x00100000 // Add values
#define SUB              0x00200000 // Substract values
#define MUL              0x00300000 // Multiplicate values
#define DIV              0x00400000 // Divide values
#define INC              0x00500000 // Increment value
#define DEC              0x00600000 // Decrement value
#define IDIV             0x00700000 // Integer division
#define MOD              0x00800000 // Divide two numbers and returns only the remainder

#define NEG              0x00900000 // Negate
#define NOT              0x00A00000 // Logical negation

// Register ops. /////// 0x-X------ /////////////////////////////////////////////////////////////////
#define MOV              0x01000000 // Move value (register-to-register and stack-to-register)
#define MOVIINT          0x02000000 // Move indirect ARRAY to register
#define MOVISTR          0x03000000 // Move indirect HASH  to register
#define IMOVINT          0x04000000 // Move register to indirect ARRAY
#define IMOVSTR          0x05000000 // Move register to indirect HASH
#define MOVSIZE          0x06000000 // Move size of array or hash to register
#define MOVIREG          0x07000000 // Move indirect ARRAY from REGISTER pointed by REGISTER to another register

// Conditional ops. //// 0xX------X //////////////////////////////////////////////////////////////////
#define CMP              0x10000000 // Compare values
#define JXX              0x20000000 // Conditional jump, generic
#define JN               0x20000001 // Jump if NOT Equal
#define JL               0x20000002 // Jump if Lesser
#define JG               0x20000004 // Jump if Greater
#define JE               0x20000008 // Jump if Equal
#define JLE              (JL | JE)  // Jump if Less OR Equal
#define JGE              (JG | JE)  // Jump if Greater OR Equal
#define JODD             0x20000010 // Jump if Result is ODD
#define JEVEN            0x20000000 // Jump if Result is Even

#define EXIST            0x30000000 // Check existence of operand

// Comparison flags //// 0x0000000X //////////////////////////////////////////////////////////////////
#define FL_NE            0x00000001 // Not Equal
#define FL_LT            0x00000002 // Less
#define FL_GT            0x00000004 // Greater
#define FL_EQ            0x00000008 // Equal
#define FL_ODD           0x00000010 // First comparison result is Odd

// Sources ///////////// 0x-------X //////////////////////////////////////////////////////////////////
#define ARG_SRC_AR       0x00000000 // AR is source register
#define ARG_SRC_BR       0x00000001 // BR is source register
#define ARG_SRC_CR       0x00000002 // CR is source register
#define ARG_SRC_DR       0x00000003 // DR is source register
#define ARG_SRC_ER       0x00000004 // ER is source register
#define ARG_SRC_FR       0x00000005 // FR is source register
#define ARG_SRC_GR       0x00000006 // GR is source register
#define ARG_SRC_HR       0x00000007 // HR is source register

#define ARG_SRC_LASTREG  ARG_SRC_HR // Last regsiter

#define ARG_SRC_INT      0x00000008 // Integer value from static data segment is source
#define ARG_SRC_FLOAT    0x00000009 // Float value from static data segment is source
#define ARG_SRC_STR      0x0000000A // String value from static text segment is source
#define ARG_SRC_STACK    0x0000000B // Stack value is source
#define ARG_SRC_IND_VAL  0x0000000C // Stack value is indirect array value is source
#define ARG_SRC_IND_STR  0x0000000D // Stack value is indirect hash value is source

// Destinations //////// 0x-----X-- /////////////////////////////////////////////////////
#define ARG_DST_AR       0x00000000 // AR is destination register
#define ARG_DST_BR       0x00000100 // BR is destination register
#define ARG_DST_CR       0x00000200 // CR is destination register
#define ARG_DST_DR       0x00000300 // DR is destination register
#define ARG_DST_ER       0x00000400 // DR is destination register
#define ARG_DST_FR       0x00000500 // DR is destination register
#define ARG_DST_GR       0x00000600 // DR is destination register
#define ARG_DST_HR       0x00000700 // DR is destination register

#define ARG_DST_LASTREG  ARG_DST_HR // Last regsiter

#define ARG_DST_STACK    0x00000800 // Top of Stack value is destination
#define ARG_DST_IND_VAL  0x00000900 // Stack value is indirect array value is destination
#define ARG_DST_IND_STR  0x00000A00 // Stack value is indirect hash value is destination


#define SYSCALL_PARAMS(x, y) ( ((UINT_32(x) << 16) & 0xFFFF0000) + (UINT_32(y) & 0x0000FFFF) )

#endif // _VM_OPCODES_H__
// End.
